package org.sourceforge.pasclipse.actions;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.sourceforge.pasclipse.PasclipsePlugin;
import org.sourceforge.pasclipse.properties.PasclipsePropertyPage;

/**
 * @author Sirkillfur
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class CallCompiler implements IRunnableWithProgress {

	private ISelection selection = null;
	private Shell shell = null;
	private String pathToCompiler = null;
	private String cmdLineOptions = null;

	/**
	 * Constructor for CallCompiler.
	 */
	public CallCompiler() {
		super();
	}

	/**
	 * @see org.eclipse.jface.operation.IRunnableWithProgress#run(IProgressMonitor)
	 */
	public void run(IProgressMonitor monitor)
		throws InvocationTargetException, InterruptedException {

		monitor.beginTask("PASCAL compiler launched by Pasclipse ...", 10);

		monitor.worked(1);
		monitor.subTask("Load preferences ...");

		IPreferenceStore store =
			PasclipsePlugin.getDefault().getPreferenceStore();
		pathToCompiler = store.getString("pathPreference");
		cmdLineOptions = store.getString("stringPreference");

		if (pathToCompiler.equals("C:\\Program Files\\Pascal Compiler EXE")
			&& cmdLineOptions.equals("Options here!")) {
			MessageDialog.openError(
				shell,
				"Pasclipse: Very Stupid Compilation Error by User",
				"Incorrect preferences.\nCheck preferences: Window > Preferences > Pasclipse");
			monitor.setCanceled(true);
			return;
		}
		if (pathToCompiler.equals("") && cmdLineOptions.equals("")) {
			MessageDialog.openError(
				shell,
				"Pasclipse: Very Stupid Compilation Error by User",
				"Incorrect preferences.\nCheck preferences: Window > Preferences > Pasclipse");
			monitor.setCanceled(true);
			return;
		}
		if (!pathToCompiler.endsWith("exe")
			&& !pathToCompiler.endsWith("EXE")) {
			MessageDialog.openError(
				shell,
				"Pasclipse: Very Stupid Compilation Error by User",
				"Incorrect compiler path (file not a .exe file).\nCheck preferences: Window > Preferences > Pasclipse");
			monitor.setCanceled(true);
			return;
		}

		monitor.worked(1);
		monitor.subTask(
			"Preferences loaded. Found compiler: " + pathToCompiler);

		//check file and launch compiler
		checkFile(monitor);

		if (!monitor.isCanceled())
			monitor.done();

	}

	private void checkFile(IProgressMonitor monitor) {
		monitor.worked(1);
		monitor.subTask("Check selection ...");
		if (selection instanceof IStructuredSelection) {
			IStructuredSelection structuredSelection =
				(IStructuredSelection) selection;
			if (structuredSelection.size() == 1) {
				Object selectedResource = structuredSelection.getFirstElement();
				if (selectedResource instanceof IFile) {
					String filename = ((IFile) selectedResource).getName();
					String filePath =
						((IFile) selectedResource).getLocation().toOSString();
					monitor.worked(1);
					monitor.subTask("Check selected file " + filename + " ...");
					try {
						if (((IFile) selectedResource)
							.getFileExtension()
							.equals("pas")) {
							monitor.worked(1);
							monitor.subTask("File is a .pas file...");
							String fileCmdLineOptions = ((IFile) selectedResource).getPersistentProperty(new QualifiedName("", PasclipsePropertyPage.CMDLN_PROPERTY));
							launchCompiler(monitor, filePath, fileCmdLineOptions);
						} else {
							MessageDialog.openError(
								shell,
								"Pasclipse: Non Fatal Compilation Error by User",
								"File "
									+ filename
									+ " not a .pas file! Compilation aborted!");
							monitor.setCanceled(true);
							return;
						}
					} catch (Exception e) {
						MessageDialog.openError(
							shell,
							"Pasclipse: Fatal Compilation Error by Eclipse",
							"Failed to launch compiler with file "
								+ filename
								+ "\nError: "
								+ e.toString()
								+ "\nPossible action: Check path to compiler and its options.\nPreferences Dialog: Window > Preferences > Pasclipse");
						monitor.setCanceled(true);
						return;
					}
				} else {
					MessageDialog.openError(
						shell,
						"Pasclipse: Non Fatal Compilation Error by User",
						"Wrong selection. Compilation not started. Resource must be a file.");
					monitor.setCanceled(true);
					return;
				}
			}
		}
	}

	private void launchCompiler(
		IProgressMonitor monitor,
		String filePath,
		String fileCmdLineOptions)
		throws Exception {
		Runtime rt = Runtime.getRuntime();
		Process compileJob = rt.exec("cmd");
		compileJob.getOutputStream().flush();
		compileJob.getOutputStream().write(
			("\n"
				+ pathToCompiler
				+ " "
				+ cmdLineOptions
				+ " "
				+ filePath
				+ " "
				+ fileCmdLineOptions
				+ "\nexit\n")
				.getBytes());
		compileJob.getOutputStream().flush();
		BufferedInputStream compilerBufferedOut =
			new BufferedInputStream(compileJob.getInputStream());
		InputStreamReader reader = new InputStreamReader(compilerBufferedOut);
		BufferedReader compilerMsgs = new BufferedReader(reader);
		monitor.worked(1);
		while (compilerMsgs.readLine() != null) {
			String temp = compilerMsgs.readLine();
			//System.out.println(temp);
			if (temp.startsWith("Fatal") || temp.startsWith("Error")) {
				MessageDialog.openError(
					shell,
					"Pasclipse: Fatal Compilation Error by Pascal Compiler",
					temp);
				monitor.setCanceled(true);
				return;
			} else {
				monitor.subTask("Pascal Compiler: " + temp);
			}
		}
		monitor.worked(1);
	}

	/**
	 * Sets the selection.
	 * @param selection The selection to set
	 */
	public void setSelection(ISelection selection) {
		this.selection = selection;
	}

	/**
	 * Sets the shell.
	 * @param shell The shell to set
	 */
	public void setShell(Shell shell) {
		this.shell = shell;
	}

}
