package org.sourceforge.pasclipse.editors;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextDoubleClickStrategy;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;

public class PascalConfiguration extends SourceViewerConfiguration {
	private PascalDoubleClickStrategy doubleClickStrategy;
	private PascalTagScanner tagScanner;
	private PascalScanner scanner;
	private ColorManager colorManager;

	public PascalConfiguration(ColorManager colorManager) {
		this.colorManager = colorManager;
	}
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return new String[] {
			IDocument.DEFAULT_CONTENT_TYPE,
			PascalPartitionScanner.PAS_COMMENT,
			PascalPartitionScanner.PAS_MOD };
	}
	public ITextDoubleClickStrategy getDoubleClickStrategy(
		ISourceViewer sourceViewer,
		String contentType) {
		if (doubleClickStrategy == null)
			doubleClickStrategy = new PascalDoubleClickStrategy();
		return doubleClickStrategy;
	}

	protected PascalScanner getXMLScanner() {
		if (scanner == null) {
			scanner = new PascalScanner(colorManager);
			scanner.setDefaultReturnToken(
				new Token(
					new TextAttribute(colorManager.getColor(IPasclipseColorConstants.DEFAULT))));
		}
		return scanner;
	}
	protected PascalTagScanner getXMLTagScanner() {
		if (tagScanner == null) {
			tagScanner = new PascalTagScanner(colorManager);
			tagScanner.setDefaultReturnToken(
				new Token(new TextAttribute(colorManager.getColor(IPasclipseColorConstants.TAG))));
		}
		return tagScanner;
	}

	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {
		PresentationReconciler reconciler = new PresentationReconciler();

		DefaultDamagerRepairer dr = new DefaultDamagerRepairer(getXMLTagScanner());
		reconciler.setDamager(dr, PascalPartitionScanner.PAS_MOD);
		reconciler.setRepairer(dr, PascalPartitionScanner.PAS_MOD);

		dr = new DefaultDamagerRepairer(getXMLScanner());
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

		NonRuleBasedDamagerRepairer ndr =
			new NonRuleBasedDamagerRepairer(
				new TextAttribute(colorManager.getColor(IPasclipseColorConstants.COMMENT)));
		reconciler.setDamager(ndr, PascalPartitionScanner.PAS_COMMENT);
		reconciler.setRepairer(ndr, PascalPartitionScanner.PAS_COMMENT);

		return reconciler;
	}

}