package org.sourceforge.pasclipse.editors;

import org.eclipse.jface.text.rules.*;
import org.eclipse.jface.text.*;
import org.sourceforge.pasclipse.editors.IPasclipseColorConstants;
import org.sourceforge.pasclipse.editors.PascalWhitespaceDetector;

public class PascalScanner extends RuleBasedScanner {

	/**
	 * @author Sirkillfur
	 *
	 * To change this generated comment edit the template variable "typecomment":
	 * Window>Preferences>Java>Templates.
	 * To enable and disable the creation of type comments go to
	 * Window>Preferences>Java>Code Generation.
	 */
	public class ReservedWords implements IWordDetector {

		private int counter = 0;

		private String[] words =
			{   "and",
				"array",
				"begin",
				"case",
				"const",
				"div",
				"do",
				"downto",
				"else",
				"end",
				"file",
				"for",
				"function",
				"goto",
				"if",
				"in",
				"label",
				"mod",
				"nil",
				"not",
				"of",
				"or",
				"packed",
				"procedure",
				"program",
				"record",
				"repeat",
				"set",
				"then",
				"to",
				"type",
				"until",
				"var",
				"while",
				"with" };

		/**
		 * Constructor for ReservedWords.
		 */
		public ReservedWords() {
			super();
		}

		/**
		 * @see org.eclipse.jface.text.rules.IWordDetector#isWordStart(char)
		 */
		public boolean isWordStart(char c) {
			for (int i = 0; i < words.length; i++) {
				counter = i;
				//System.out.println("WordStart ("+i+"): " + words[i]);
				if (words[i].startsWith(String.valueOf(c)))
					return true;
			}
			return false;
		}
		/**
		 * @see org.eclipse.jface.text.rules.IWordDetector#isWordPart(char)
		 */
		public boolean isWordPart(char c) {
			//System.out.println("WordPart ("+counter+"): " + words[counter]);
			return (words[counter].indexOf(String.valueOf(c)) != -1);
		}

		/**
		 * Returns the words.
		 * @return String[]
		 */
		public String[] getWords() {
			return words;
		}

	}
	public class Identifiers implements IWordDetector {

		private int counter = 0;

		private String[] words =
			{
				"ABS",
				"ARCTAN",
				"BOOLEAN",
				"CHAR",
				"CHR",
				"COS",
				"EOF",
				"EOLN",
				"EXP",
				"FALSE",
				"GET",
				"IN",
				"INPUT",
				"INTEGER",
				"MAXINT",
				"NEW",
				"ODD",
				"OUTPUT",
				"PACK",
				"PAGE",
				"PUT",
				"READ",
				"READLN",
				"REAL",
				"RESET",
				"REWRITE",
				"SIN",
				"SQR",
				"SQRT",
				"SUCC",
				"TRUE",
				"TRUNC",
				"WRITE",
				"WRITELN" };

		/**
		 * Constructor for ReservedWords.
		 */
		public Identifiers() {
			super();
		}

		/**
		 * @see org.eclipse.jface.text.rules.IWordDetector#isWordStart(char)
		 */
		public boolean isWordStart(char c) {

			for (int i = 0; i < words.length; i++) {
				counter = i;
				if (words[i].startsWith(String.valueOf(c))
					&& String.valueOf(c).equals(String.valueOf(c).toUpperCase()))
					return true;
			}
			return false;
		}
		/**
		 * @see org.eclipse.jface.text.rules.IWordDetector#isWordPart(char)
		 */
		public boolean isWordPart(char c) {
			if (words[counter].indexOf(String.valueOf(c)) != -1
				&& String.valueOf(c).equals(String.valueOf(c).toUpperCase()))
				return true;
			return false;
		}

		/**
		 * Returns the words.
		 * @return String[]
		 */
		public String[] getWords() {
			return words;
		}
	}
	public class Operators implements IWordDetector {

		private String[] words = { "*", "/", "-", "+", "=", ":", ",", ";", "<", ">","@","$"};

		/**
		 * Constructor for ReservedWords.
		 */
		public Operators() {
			super();
		}

		/**
		 * @see org.eclipse.jface.text.rules.IWordDetector#isWordStart(char)
		 */
		public boolean isWordStart(char c) {
			for (int i = 0; i < words.length; i++) {
				if (words[i].startsWith(String.valueOf(c)))
					return true;
			}
			return false;
		}
		/**
		 * @see org.eclipse.jface.text.rules.IWordDetector#isWordPart(char)
		 */
		public boolean isWordPart(char c) {
			for (int i = 0; i < words.length; i++) {
				if (words[i].indexOf(String.valueOf(c)) != -1)
					return true;
			}
			return false;
		}

		/**
		 * Returns the words.
		 * @return String[]
		 */
		public String[] getWords() {
			return words;
		}
	}
	public class Breakets implements IWordDetector {

		private String[] words = { "(", ")", "[", "]", "{", "}"};

		/**
		 * Constructor for ReservedWords.
		 */
		public Breakets() {
			super();
		}

		/**
		 * @see org.eclipse.jface.text.rules.IWordDetector#isWordStart(char)
		 */
		public boolean isWordStart(char c) {
			for (int i = 0; i < words.length; i++) {
				if (words[i].startsWith(String.valueOf(c)))
					return true;
			}
			return false;
		}
		/**
		 * @see org.eclipse.jface.text.rules.IWordDetector#isWordPart(char)
		 */
		public boolean isWordPart(char c) {
			for (int i = 0; i < words.length; i++) {
				if (words[i].indexOf(String.valueOf(c)) != -1)
					return true;
			}
			return false;
		}

		/**
		 * Returns the words.
		 * @return String[]
		 */
		public String[] getWords() {
			return words;
		}
	}
	public PascalScanner(ColorManager manager) {
		IToken procInstr =
			new Token(
				new TextAttribute(
						manager.getColor(IPasclipseColorConstants.DEFAULT)));

		IToken comment =
			new Token(
				new TextAttribute(
					manager.getColor(IPasclipseColorConstants.COMMENT)));

		IToken reserved =
			new Token(
				new TextAttribute(
					manager.getColor(IPasclipseColorConstants.TAG)));

		IToken string =
			new Token(
				new TextAttribute(
					manager.getColor(IPasclipseColorConstants.STRING)));
		IToken numbers =
			new Token(
				new TextAttribute(
					manager.getColor(IPasclipseColorConstants.STRING)));
		IToken operators =
			new Token(
				new TextAttribute(
					manager.getColor(IPasclipseColorConstants.OPER)));
		
		
		IRule[] rules = new IRule[11];
		//Add comment rules
		rules[0] = new SingleLineRule("{$", "}", procInstr);
		rules[1] = new MultiLineRule("(*", "*)", comment);
		rules[2] = new MultiLineRule("/*", "*/", comment);
		rules[3] = new MultiLineRule("{", "}", comment);
		rules[4] = new SingleLineRule("'", "'", string);
		// Add generic whitespace rule.
		rules[5] = new WhitespaceRule(new PascalWhitespaceDetector());
		// add word rules
		rules[6] = new WordRule(new ReservedWords(), reserved);
		rules[7] = new WordRule(new Identifiers(), procInstr);
		//numbers
		rules[8] = new NumberRule(numbers);
		//operators
		rules[9] = new WordRule(new Operators(), operators);
		//breakets
		rules[10] = new WordRule(new Breakets(), operators);
		
		setRules(rules);
	}
}
